"""
The tokenize package allows us to return nicely contextual error messages that
indicate where in a document an error has occurred.

It does this by parsing JSON or YAML and returning items that store both the
value and the position of each element in the parsed datastructure.
"""
