<!DOCTYPE html>
<html>
  <head>
    <title>{{ document.title|default('API Documentation', True) }}</title>
    <!-- needed for adaptive design -->
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link href="https://fonts.googleapis.com/css?family=Montserrat:300,400,700|Roboto:300,400,700" rel="stylesheet">

    <!--
    ReDoc doesn't change outer page styles
    -->
    <style>
      body {
        margin: 0;
        padding: 0;
      }
    </style>
  </head>
  <body>
    <redoc spec-url='{{ schema_url }}'></redoc>
    <script src="{{ static_url('js/redoc.standalone.js') }}"> </script>
  </body>
</html>
